@@ -1,35 +1,55 @@
 from __future__ import annotations
 
 from collections.abc import Callable
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING
 
 from docutils import nodes
 from docutils.statemachine import StringList
-from docutils.utils import Reporter, assemble_option_dict
+from docutils.utils import assemble_option_dict
 
-from sphinx.ext.autodoc import Documenter, Options
+from sphinx.ext.autodoc import Options
 from sphinx.util import logging
 from sphinx.util.docutils import SphinxDirective, switch_source_input
 from sphinx.util.parsing import nested_parse_to_nodes
 
 if TYPE_CHECKING:
+    from typing import Any
+
     from docutils.nodes import Node
     from docutils.parsers.rst.states import RSTState
+    from docutils.utils import Reporter
 
     from sphinx.config import Config
     from sphinx.environment import BuildEnvironment
+    from sphinx.ext.autodoc import Documenter
 
 logger = logging.getLogger(__name__)
 
 
 # common option names for autodoc directives
-AUTODOC_DEFAULT_OPTIONS = ['members', 'undoc-members', 'inherited-members',
-                           'show-inheritance', 'private-members', 'special-members',
-                           'ignore-module-all', 'exclude-members', 'member-order',
-                           'imported-members', 'class-doc-from', 'no-value']
+AUTODOC_DEFAULT_OPTIONS = [
+    'members',
+    'undoc-members',
+    'no-index',
+    'no-index-entry',
+    'inherited-members',
+    'show-inheritance',
+    'private-members',
+    'special-members',
+    'ignore-module-all',
+    'exclude-members',
+    'member-order',
+    'imported-members',
+    'class-doc-from',
+    'no-value',
+]
 
-AUTODOC_EXTENDABLE_OPTIONS = ['members', 'private-members', 'special-members',
-                              'exclude-members']
+AUTODOC_EXTENDABLE_OPTIONS = frozenset({
+    'members',
+    'private-members',
+    'special-members',
+    'exclude-members',
+})
 
 
 class DummyOptionSpec(dict[str, Callable[[str], str]]):
@@ -46,8 +66,14 @@ class DummyOptionSpec(dict[str, Callable[[str], str]]):
 class DocumenterBridge:
     """A parameters container for Documenters."""
 
-    def __init__(self, env: BuildEnvironment, reporter: Reporter | None, options: Options,
-                 lineno: int, state: Any) -> None:
+    def __init__(
+        self,
+        env: BuildEnvironment,
+        reporter: Reporter | None,
+        options: Options,
+        lineno: int,
+        state: Any,
+    ) -> None:
         self.env = env
         self._reporter = reporter
         self.genopt = options
@@ -58,7 +84,7 @@ class DocumenterBridge:
 
 
 def process_documenter_options(
-    documenter: type[Documenter], config: Config, options: dict[str, str],
+    documenter: type[Documenter], config: Config, options: dict[str, str]
 ) -> Options:
     """Recognize options of Documenter from user input."""
     default_options = config.autodoc_default_options
@@ -83,8 +109,9 @@ def process_documenter_options(
     return Options(assemble_option_dict(options.items(), documenter.option_spec))
 
 
-def parse_generated_content(state: RSTState, content: StringList, documenter: Documenter,
-                            ) -> list[Node]:
+def parse_generated_content(
+    state: RSTState, content: StringList, documenter: Documenter
+) -> list[Node]:
     """Parse an item of content generated by Documenter."""
     with switch_source_input(state, content):
         if documenter.titles_allowed:
@@ -115,26 +142,35 @@ class AutodocDirective(SphinxDirective):
 
         try:
             source, lineno = reporter.get_source_and_line(  # type: ignore[attr-defined]
-                self.lineno)
+                self.lineno
+            )
         except AttributeError:
             source, lineno = (None, None)
         logger.debug('[autodoc] %s:%s: input:\n%s', source, lineno, self.block_text)
 
         # look up target Documenter
         objtype = self.name[4:]  # strip prefix (auto-).
-        doccls = self.env.app.registry.documenters[objtype]
+        doccls = self.env._registry.documenters[objtype]
 
         # process the options with the selected documenter's option_spec
         try:
-            documenter_options = process_documenter_options(doccls, self.config, self.options)
+            documenter_options = process_documenter_options(
+                doccls, self.config, self.options
+            )
         except (KeyError, ValueError, TypeError) as exc:
             # an option is either unknown or has a wrong type
-            logger.error('An option to %s is either unknown or has an invalid value: %s',
-                         self.name, exc, location=(self.env.docname, lineno))
+            logger.error(  # NoQA: TRY400
+                'An option to %s is either unknown or has an invalid value: %s',
+                self.name,
+                exc,
+                location=(self.env.docname, lineno),
+            )
             return []
 
         # generate the output
-        params = DocumenterBridge(self.env, reporter, documenter_options, lineno, self.state)
+        params = DocumenterBridge(
+            self.env, reporter, documenter_options, lineno, self.state
+        )
         documenter = doccls(params, self.arguments[0])
         documenter.generate(more_content=self.content)
         if not params.result:
