@@ -4,16 +4,17 @@ import os
 import shutil
 import sys
 import warnings
-from typing import IO, TYPE_CHECKING, Any
+from typing import TYPE_CHECKING
 
 from sphinx.deprecation import RemovedInSphinx90Warning
 
 if TYPE_CHECKING:
     import builtins
     from collections.abc import Callable
+    from typing import IO, Any
 
 warnings.warn(
-    "'sphinx.testing.path' is deprecated. " "Use 'os.path' or 'pathlib' instead.",
+    "'sphinx.testing.path' is deprecated. Use 'os.path' or 'pathlib' instead.",
     RemovedInSphinx90Warning,
     stacklevel=2,
 )
@@ -32,57 +33,41 @@ def getumask() -> int:
 UMASK = getumask()
 
 
-class path(str):
-    """
-    Represents a path which behaves like a string.
-    """
+class path(str):  # NoQA: FURB189
+    """Represents a path which behaves like a string."""
 
     __slots__ = ()
 
     @property
     def parent(self) -> path:
-        """
-        The name of the directory the file or directory is in.
-        """
+        """The name of the directory the file or directory is in."""
         return self.__class__(os.path.dirname(self))
 
     def basename(self) -> str:
         return os.path.basename(self)
 
     def abspath(self) -> path:
-        """
-        Returns the absolute path.
-        """
+        """Returns the absolute path."""
         return self.__class__(os.path.abspath(self))
 
     def isabs(self) -> bool:
-        """
-        Returns ``True`` if the path is absolute.
-        """
+        """Returns ``True`` if the path is absolute."""
         return os.path.isabs(self)
 
     def isdir(self) -> bool:
-        """
-        Returns ``True`` if the path is a directory.
-        """
+        """Returns ``True`` if the path is a directory."""
         return os.path.isdir(self)
 
     def isfile(self) -> bool:
-        """
-        Returns ``True`` if the path is a file.
-        """
+        """Returns ``True`` if the path is a file."""
         return os.path.isfile(self)
 
     def islink(self) -> bool:
-        """
-        Returns ``True`` if the path is a symbolic link.
-        """
+        """Returns ``True`` if the path is a symbolic link."""
         return os.path.islink(self)
 
     def ismount(self) -> bool:
-        """
-        Returns ``True`` if the path is a mount point.
-        """
+        """Returns ``True`` if the path is a mount point."""
         return os.path.ismount(self)
 
     def rmtree(
@@ -90,8 +75,7 @@ class path(str):
         ignore_errors: bool = False,
         onerror: Callable[[Callable[..., Any], str, Any], object] | None = None,
     ) -> None:
-        """
-        Removes the file or directory and any files or directories it may
+        """Removes the file or directory and any files or directories it may
         contain.
 
         :param ignore_errors:
@@ -108,8 +92,7 @@ class path(str):
         shutil.rmtree(self, ignore_errors=ignore_errors, onerror=onerror)
 
     def copytree(self, destination: str, symlinks: bool = False) -> None:
-        """
-        Recursively copy a directory to the given `destination`. If the given
+        """Recursively copy a directory to the given `destination`. If the given
         `destination` does not exist it will be created.
 
         :param symlinks:
@@ -130,8 +113,7 @@ class path(str):
                     os.chmod(os.path.join(root, name), 0o644 & ~UMASK)
 
     def movetree(self, destination: str) -> None:
-        """
-        Recursively move the file or directory to the given `destination`
+        """Recursively move the file or directory to the given `destination`
         similar to the  Unix "mv" command.
 
         If the `destination` is a file it may be overwritten depending on the
@@ -142,47 +124,36 @@ class path(str):
     move = movetree
 
     def unlink(self) -> None:
-        """
-        Removes a file.
-        """
+        """Removes a file."""
         os.unlink(self)
 
     def stat(self) -> Any:
-        """
-        Returns a stat of the file.
-        """
+        """Returns a stat of the file."""
         return os.stat(self)
 
     def utime(self, arg: Any) -> None:
         os.utime(self, arg)
 
     def open(self, mode: str = 'r', **kwargs: Any) -> IO[str]:
-        return open(self, mode, **kwargs)  # NoQA: SIM115
+        return open(self, mode, **kwargs)
 
     def write_text(self, text: str, encoding: str = 'utf-8', **kwargs: Any) -> None:
-        """
-        Writes the given `text` to the file.
-        """
+        """Writes the given `text` to the file."""
         with open(self, 'w', encoding=encoding, **kwargs) as f:
             f.write(text)
 
     def read_text(self, encoding: str = 'utf-8', **kwargs: Any) -> str:
-        """
-        Returns the text in the file.
-        """
+        """Returns the text in the file."""
         with open(self, encoding=encoding, **kwargs) as f:
             return f.read()
 
     def read_bytes(self) -> builtins.bytes:
-        """
-        Returns the bytes in the file.
-        """
+        """Returns the bytes in the file."""
         with open(self, mode='rb') as f:
             return f.read()
 
     def write_bytes(self, bytes: bytes, append: bool = False) -> None:
-        """
-        Writes the given `bytes` to the file.
+        """Writes the given `bytes` to the file.
 
         :param append:
             If ``True`` given `bytes` are added at the end of the file.
@@ -195,28 +166,21 @@ class path(str):
             f.write(bytes)
 
     def exists(self) -> bool:
-        """
-        Returns ``True`` if the path exist.
-        """
+        """Returns ``True`` if the path exist."""
         return os.path.exists(self)
 
     def lexists(self) -> bool:
-        """
-        Returns ``True`` if the path exists unless it is a broken symbolic
+        """Returns ``True`` if the path exists unless it is a broken symbolic
         link.
         """
         return os.path.lexists(self)
 
     def makedirs(self, mode: int = 0o777, exist_ok: bool = False) -> None:
-        """
-        Recursively create directories.
-        """
+        """Recursively create directories."""
         os.makedirs(self, mode, exist_ok=exist_ok)
 
     def joinpath(self, *args: Any) -> path:
-        """
-        Joins the path with the argument given and returns the result.
-        """
+        """Joins the path with the argument given and returns the result."""
         return self.__class__(os.path.join(self, *map(self.__class__, args)))
 
     def listdir(self) -> list[str]:
