@@ -11,24 +11,24 @@ import pytest
 from docutils import nodes
 
 from sphinx import addnodes
+from sphinx._cli.util.errors import strip_escape_sequences
 from sphinx.builders.html import INVENTORY_FILENAME
 from sphinx.config import Config
 from sphinx.errors import ConfigError
-from sphinx.ext.intersphinx import (
-    inspect_main,
-    load_mappings,
-    missing_reference,
-    validate_intersphinx_mapping,
-)
 from sphinx.ext.intersphinx import setup as intersphinx_setup
+from sphinx.ext.intersphinx._cli import inspect_main
 from sphinx.ext.intersphinx._load import (
     _fetch_inventory,
     _fetch_inventory_group,
     _get_safe_url,
+    _InvConfig,
     _strip_basic_auth,
+    load_mappings,
+    validate_intersphinx_mapping,
 )
+from sphinx.ext.intersphinx._resolve import missing_reference
 from sphinx.ext.intersphinx._shared import _IntersphinxProject
-from sphinx.util.console import strip_colors
+from sphinx.util.inventory import _Inventory, _InventoryItem
 
 from tests.test_util.intersphinx_data import (
     INVENTORY_V2,
@@ -40,8 +40,12 @@ from tests.utils import http_server
 if TYPE_CHECKING:
     from typing import NoReturn
 
+    from sphinx.ext.intersphinx._shared import InventoryCacheEntry
+    from sphinx.testing.util import SphinxTestApp
+    from sphinx.util.typing import Inventory
 
-class FakeList(list):
+
+class FakeList(list[str]):
     def __iter__(self) -> NoReturn:
         raise NotImplementedError
 
@@ -67,35 +71,38 @@ def set_config(app, mapping):
     app.config.intersphinx_mapping = mapping.copy()
     app.config.intersphinx_cache_limit = 0
     app.config.intersphinx_disabled_reftypes = []
+    app.config.intersphinx_resolve_self = ''
+    app.config.intersphinx_timeout = None
 
 
 @mock.patch('sphinx.ext.intersphinx._load.InventoryFile')
-@mock.patch('sphinx.ext.intersphinx._load._read_from_url')
+@mock.patch('sphinx.ext.intersphinx._load.requests.get')
 @pytest.mark.sphinx('html', testroot='root')
-def test_fetch_inventory_redirection(_read_from_url, InventoryFile, app):  # NoQA: PT019
+def test_fetch_inventory_redirection(get_request, InventoryFile, app):
+    mocked_get = get_request.return_value.__enter__.return_value
     intersphinx_setup(app)
-    _read_from_url().readline.return_value = b'# Sphinx inventory version 2'
+    mocked_get.content = b'# Sphinx inventory version 2'
 
     # same uri and inv, not redirected
-    _read_from_url().url = 'https://hostname/' + INVENTORY_FILENAME
+    mocked_get.url = 'https://hostname/' + INVENTORY_FILENAME
     _fetch_inventory(
         target_uri='https://hostname/',
         inv_location='https://hostname/' + INVENTORY_FILENAME,
-        config=app.config,
+        config=_InvConfig.from_config(app.config),
         srcdir=app.srcdir,
     )
     assert 'intersphinx inventory has moved' not in app.status.getvalue()
-    assert InventoryFile.load.call_args[0][1] == 'https://hostname/'
+    assert InventoryFile.loads.call_args[1]['uri'] == 'https://hostname/'
 
     # same uri and inv, redirected
     app.status.seek(0)
     app.status.truncate(0)
-    _read_from_url().url = 'https://hostname/new/' + INVENTORY_FILENAME
+    mocked_get.url = 'https://hostname/new/' + INVENTORY_FILENAME
 
     _fetch_inventory(
         target_uri='https://hostname/',
         inv_location='https://hostname/' + INVENTORY_FILENAME,
-        config=app.config,
+        config=_InvConfig.from_config(app.config),
         srcdir=app.srcdir,
     )
     assert app.status.getvalue() == (
@@ -103,31 +110,31 @@ def test_fetch_inventory_redirection(_read_from_url, InventoryFile, app):  # NoQ
         'https://hostname/%s -> https://hostname/new/%s\n'
         % (INVENTORY_FILENAME, INVENTORY_FILENAME)
     )
-    assert InventoryFile.load.call_args[0][1] == 'https://hostname/new'
+    assert InventoryFile.loads.call_args[1]['uri'] == 'https://hostname/new'
 
     # different uri and inv, not redirected
     app.status.seek(0)
     app.status.truncate(0)
-    _read_from_url().url = 'https://hostname/new/' + INVENTORY_FILENAME
+    mocked_get.url = 'https://hostname/new/' + INVENTORY_FILENAME
 
     _fetch_inventory(
         target_uri='https://hostname/',
         inv_location='https://hostname/new/' + INVENTORY_FILENAME,
-        config=app.config,
+        config=_InvConfig.from_config(app.config),
         srcdir=app.srcdir,
     )
     assert 'intersphinx inventory has moved' not in app.status.getvalue()
-    assert InventoryFile.load.call_args[0][1] == 'https://hostname/'
+    assert InventoryFile.loads.call_args[1]['uri'] == 'https://hostname/'
 
     # different uri and inv, redirected
     app.status.seek(0)
     app.status.truncate(0)
-    _read_from_url().url = 'https://hostname/other/' + INVENTORY_FILENAME
+    mocked_get.url = 'https://hostname/other/' + INVENTORY_FILENAME
 
     _fetch_inventory(
         target_uri='https://hostname/',
         inv_location='https://hostname/new/' + INVENTORY_FILENAME,
-        config=app.config,
+        config=_InvConfig.from_config(app.config),
         srcdir=app.srcdir,
     )
     assert app.status.getvalue() == (
@@ -135,7 +142,7 @@ def test_fetch_inventory_redirection(_read_from_url, InventoryFile, app):  # NoQ
         'https://hostname/new/%s -> https://hostname/other/%s\n'
         % (INVENTORY_FILENAME, INVENTORY_FILENAME)
     )
-    assert InventoryFile.load.call_args[0][1] == 'https://hostname/'
+    assert InventoryFile.loads.call_args[1]['uri'] == 'https://hostname/'
 
 
 @pytest.mark.sphinx('html', testroot='root')
@@ -155,13 +162,13 @@ def test_missing_reference(tmp_path, app):
     # load the inventory and check if it's done correctly
     validate_intersphinx_mapping(app, app.config)
     load_mappings(app)
-    inv = app.env.intersphinx_inventory
+    inv: Inventory = app.env.intersphinx_inventory
 
-    assert inv['py:module']['module2'] == (
-        'foo',
-        '2.0',
-        'https://docs.python.org/foo.html#module-module2',
-        '-',
+    assert inv['py:module']['module2'] == _InventoryItem(
+        project_name='foo',
+        project_version='2.0',
+        uri='https://docs.python.org/foo.html#module-module2',
+        display_name='-',
     )
 
     # check resolution when a target is found
@@ -501,7 +508,7 @@ def test_inventory_not_having_version(tmp_path, app):
 
 
 @pytest.mark.sphinx('html', testroot='root')
-def test_validate_intersphinx_mapping_warnings(app):
+def test_validate_intersphinx_mapping_warnings(app: SphinxTestApp) -> None:
     """Check warnings in :func:`sphinx.ext.intersphinx.validate_intersphinx_mapping`."""
     bad_intersphinx_mapping = {
         '':                 ('789.example', None),     # invalid project name (value)
@@ -532,7 +539,7 @@ def test_validate_intersphinx_mapping_warnings(app):
         match=r'^Invalid `intersphinx_mapping` configuration \(16 errors\).$',
     ):
         validate_intersphinx_mapping(app, app.config)
-    warnings = strip_colors(app.warning.getvalue()).splitlines()
+    warnings = strip_escape_sequences(app.warning.getvalue()).splitlines()
     assert len(warnings) == len(bad_intersphinx_mapping) - 3
     assert warnings == [
         "ERROR: Invalid intersphinx project identifier `''` in intersphinx_mapping. Project identifiers must be non-empty strings.",
@@ -620,7 +627,7 @@ class TestStripBasicAuth:
         assert actual == expected
 
 
-def test_getsafeurl_authed():
+def test_getsafeurl_authed() -> None:
     """_get_safe_url() with a url with basic auth"""
     url = 'https://user:12345@domain.com/project/objects.inv'
     expected = 'https://user@domain.com/project/objects.inv'
@@ -628,7 +635,7 @@ def test_getsafeurl_authed():
     assert actual == expected
 
 
-def test_getsafeurl_authed_having_port():
+def test_getsafeurl_authed_having_port() -> None:
     """_get_safe_url() with a url with basic auth having port"""
     url = 'https://user:12345@domain.com:8080/project/objects.inv'
     expected = 'https://user@domain.com:8080/project/objects.inv'
@@ -636,7 +643,7 @@ def test_getsafeurl_authed_having_port():
     assert actual == expected
 
 
-def test_getsafeurl_unauthed():
+def test_getsafeurl_unauthed() -> None:
     """_get_safe_url() with a url without basic auth"""
     url = 'https://domain.com/project/objects.inv'
     expected = 'https://domain.com/project/objects.inv'
@@ -691,8 +698,8 @@ def test_inspect_main_url(capsys):
     assert stderr == ''
 
 
-@pytest.mark.sphinx('html', testroot='ext-intersphinx-role')
-def test_intersphinx_role(app):
+@pytest.mark.sphinx('html', testroot='ext-intersphinx-role', copy_test_root=True)
+def test_intersphinx_role(app: SphinxTestApp) -> None:
     inv_file = app.srcdir / 'inventory'
     inv_file.write_bytes(INVENTORY_V2)
     app.config.intersphinx_mapping = {
@@ -707,7 +714,7 @@ def test_intersphinx_role(app):
 
     app.build()
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
-    warnings = strip_colors(app.warning.getvalue()).splitlines()
+    warnings = strip_escape_sequences(app.warning.getvalue()).splitlines()
     index_path = app.srcdir / 'index.rst'
     assert warnings == [
         f"{index_path}:21: WARNING: role for external cross-reference not found in domain 'py': 'nope' [intersphinx.external]",
@@ -740,18 +747,14 @@ def test_intersphinx_role(app):
     assert html.format('index.html#foons') in content
 
 
-if TYPE_CHECKING:
-    from sphinx.ext.intersphinx._shared import InventoryCacheEntry
-
-
 @pytest.mark.sphinx('html', testroot='root')
 @pytest.mark.parametrize(
     ('cache_limit', 'expected_expired'),
     [
-        (5, False),
-        (1, True),
-        (0, True),
-        (-1, False),
+        (5, False),  # cache for 5 days
+        (1, True),  # cache for 1 day
+        (0, True),  # cache for 0 days
+        (-1, False),  # cache forever
     ],
 )
 def test_intersphinx_cache_limit(app, monkeypatch, cache_limit, expected_expired):
@@ -760,6 +763,7 @@ def test_intersphinx_cache_limit(app, monkeypatch, cache_limit, expected_expired
     app.config.intersphinx_mapping = {
         'inv': (url, None),
     }
+    app.config.intersphinx_timeout = None
     # load the inventory and check if it's done correctly
     intersphinx_cache: dict[str, InventoryCacheEntry] = {
         url: ('inv', 0, {}),  # Timestamp of last cache write is zero.
@@ -773,7 +777,8 @@ def test_intersphinx_cache_limit(app, monkeypatch, cache_limit, expected_expired
     # `_fetch_inventory_group` calls `_fetch_inventory`.
     # We replace it with a mock to test whether it has been called.
     # If it has been called, it means the cache had expired.
-    mock_fetch_inventory = mock.Mock(return_value=('inv', now, {}))
+    mock_fake_inventory = _Inventory({})  # must be truthy
+    mock_fetch_inventory = mock.Mock(return_value=mock_fake_inventory)
     monkeypatch.setattr(
         'sphinx.ext.intersphinx._load._fetch_inventory', mock_fetch_inventory
     )
@@ -784,7 +789,7 @@ def test_intersphinx_cache_limit(app, monkeypatch, cache_limit, expected_expired
             project=project,
             cache=intersphinx_cache,
             now=now,
-            config=app.config,
+            config=_InvConfig.from_config(app.config),
             srcdir=app.srcdir,
         )
         # If we hadn't mocked `_fetch_inventory`, it would've made
